proc GUI {dir} {
	global file resultfile genomefile dbfile flank5p flank3p drawing browser progresspage
	set genomefile [file normalize $dir/Databases/genome_hg19.ifas]
	set dbfile [file normalize $dir/Databases/mirna_hg19_mirbase20.tsv]
	set resultfile ""
	set flank5p 100
	set flank3p 100
	set drawing 1
	set browser 1
	set progresspage 0
	wm title . "miRVaS"
	set pos 1
	destroy {*}[winfo children .]
	ttk::frame .top -padding 5
	grid columnconfigure .top 1 -weight 1
	pack .top -fill both -expand yes
	foreach {variable label type} {
		file "Input variants file" openfile
		resultfile "Output file" savefile
		genomefile "Genome file" openfile
		dbfile "miRNA database file" openfile
		flank5p "Upstream flank (5p)" int
		flank3p "Downstream flank (3p)" int
		drawing "Drawing: Output html with drawings" bool
		browser "Browser: Open browser showing (drawing) results after completion" bool
	} {
		if {$type eq "bool"} {
			regexp {^([^:]+): *(.*)$} $label temp label postlabel
			ttk::label .top.label$pos -text "$label"
			ttk::checkbutton .top.entry$pos -variable $variable -text $postlabel
		} else {
			ttk::label .top.label$pos -text "$label"
			ttk::entry .top.entry$pos -textvariable $variable
		}
		set togrid [list .top.label$pos .top.entry$pos]
		if {$type in "openfile savefile"} {
			if {$type eq "openfile"} {set cmd tk_getOpenFile} else {set cmd tk_getSaveFile}
			ttk::button .top.button$pos -text "Select file" -command "set $variable \[$cmd -defaultextension {.tsv}\]"
			lappend togrid .top.button$pos
		}
		grid  {*}$togrid -sticky we
		incr pos
	}
	ttk::frame .top.buttons -padding 2
	grid .top.buttons -row $pos -column 0 -columnspan 3 -sticky nwes
	grid columnconfigure .top.buttons 0 -weight 1
	grid columnconfigure .top.buttons 1 -weight 1
	ttk::button .top.buttons.convert -text "Convert" -default active -command GUI_convert
	ttk::button .top.buttons.run -text "Start Prediction" -default active -command GUI_run
	grid  .top.buttons.run .top.buttons.convert -sticky nwes
	ttk::progressbar .top.progress -mode determinate
	grid .top.progress -sticky we -columnspan 3
	ttk::label .top.log -text ""
	grid .top.log -sticky we -columnspan 3
	update idletasks
	wm geometry . 600x[winfo height .]
}

proc GUI_convert_do {} {
	global file resultfile genomefile flank5p flank3p drawing sourcefile structfile
	destroy .convert
	catch {
		.top.progress configure -maximum 1
		.top.progress configure -value 0
		update idletasks
	}
	convertmirbase $sourcefile $resultfile $genomefile $structfile
	catch {
		.top.progress configure -value 1
		update idletasks
	}
}

proc GUI_convert {} {
	global file resultfile genomefile flank5p flank3p drawing sourcefile structfile
	if {![info exists structfile]} {set structfile {}}
	if {![info exists sourcefile]} {set sourcefile [file normalize hsa.gff3]}
	if {![info exists resultfile]} {set resultfile [file normalize hsa.tsv]}
	set pos 1
	destroy .convert
	toplevel .convert
	wm title .convert "miRVaS convert"
	ttk::frame .convert.top -padding 5
	grid columnconfigure .convert.top 1 -weight 1
	pack .convert.top -fill both -expand yes
	foreach {variable label type} {
		sourcefile "Source file (gff3)" openfile
		resultfile "Output mirna database file" savefile
		genomefile "Genome file" openfile
		structfile "Structure file" openfile
	} {
		if {$type eq "bool"} {
			regexp {^([^:]+): *(.*)$} $label temp label postlabel
			ttk::label .convert.top.label$pos -text "$label"
			ttk::checkbutton .convert.top.entry$pos -variable $variable -text $postlabel
		} else {
			ttk::label .convert.top.label$pos -text "$label"
			ttk::entry .convert.top.entry$pos -textvariable $variable
		}
		set togrid [list .convert.top.label$pos .convert.top.entry$pos]
		if {$type in "openfile savefile"} {
			if {$type eq "openfile"} {set cmd tk_getOpenFile} else {set cmd tk_getSaveFile}
			ttk::button .convert.top.button$pos -text "Select file" -command "set $variable \[$cmd -defaultextension {.tsv}\]"
			lappend togrid .convert.top.button$pos
		}
		grid  {*}$togrid -sticky we
		incr pos
	}
	ttk::frame .convert.top.buttons -padding 2
	grid .convert.top.buttons -row 7 -column 0 -columnspan 3 -sticky nwes
	grid columnconfigure .convert.top.buttons 0 -weight 1
	grid columnconfigure .convert.top.buttons 1 -weight 1
	ttk::button .convert.top.buttons.convert -text "Convert" -default active -command GUI_convert_do
	ttk::button .convert.top.buttons.cancel -text "Cancel" -default active -command {destroy .convert}
	grid .convert.top.buttons.convert  .convert.top.buttons.cancel -sticky nwes
	update idletasks
	wm geometry . 600x[winfo height .]
}

proc GUI_run {} {
	global file resultfile genomefile dbfile flank5p flank3p drawing browser genomef
	set flanksizes [list $flank5p $flank3p]
	if {$drawing} {set mirvas 2} else {set mirvas 1}
	set genecol name
	set transcriptcol transcript
	set f [gzopen $file]
	set header [tsv_open $f]
	set size 0
	while {![eof $f]} {
		set line [gets $f]
		if {$line eq ""} continue
		incr size
	}
	close $f
	if {$drawing} {set max [expr {6*$size}]} else {set max [expr {3*$size}]}
	progress_max $max 0
	if {$resultfile eq ""} {
		set useresultfile [file normalize [file dir $file]/annot_[file tail $file]]
	} else {
		set useresultfile [file normalize $resultfile]
	}
	set file [convert_if_vcf $file]
	catch {genome_close $genomef}
	annotatemir $file $genomefile $dbfile mir $useresultfile $genecol $transcriptcol $flanksizes 0 $mirvas
	if {$browser} {launchBrowser [file root $useresultfile].html}
	catch {
		.top.progress configure -value [exrp {6*$size}]
		update idletasks
	}
	log done
}

if 0 {
wm title . "test"
ttk::notebook .nb
ttk::frame .nb.tab1
.nb add .nb.tab1 -text "Main"
grid .nb
TkCreateAndGrid entry .nb.tab1.inputKnownEntry "" 1 1 no 75
TkCreateAndGrid button .nb.tab1.inputKnownBrowse "Info script" 1 2 w "" put	
}
