proc mirvas_start {mirvas file resultfile {flanksizes {}}} {
	global onestructdata genomefile dbfile
	if {$mirvas == 2} {
		file mkdir [file root $resultfile].structs
		file copy -force $::appdir/templates/setimgwidth.js [file root $resultfile].structs
		set mo [open [file root $resultfile].html.temp w]
		puts $mo {<html><head></head><body>
<style type="text/css">
.myTable { background-color:#eee; border-collapse:collapse; }
.myTable th { background-color:#000; color:white; text-align:left; }
.myTable td, .myTable th { padding:5px; border:1px solid #000; }
</style>
}
		puts $mo "<h1><a href=\"http://mirvas.bioinf.be\">miRVaS</a> results</h1>"
		puts $mo "<table>"
		puts $mo "<tr><td>Inputfile</td><td>[file tail $file]</td></tr>"
		puts $mo "<tr><td>mirnafile</td><td>[file tail $dbfile]</td></tr>"
		puts $mo "<tr><td>genomefile</td><td>[file tail $genomefile]</td></tr>"
		puts $mo "<tr><td>Creation</td><td>[clock format [clock seconds] -format %Y-%m-%d]</td></tr>"
		if {[llength $flanksizes]} {
			puts $mo "<tr><td>flanksizes</td><td>$flanksizes</td></tr>"
		}
		puts $mo "</table>"
		puts $mo "<a href=\"[file tail $resultfile]\">tab-delimited result file</a></h3>"
		puts $mo "<p>"
		puts $mo "<table class=\"myTable\">"
		puts $mo "<tr><th>variant</th><th>location</th><th>structural changes</th><th>hairpin conservation</th></tr>"
		set onestructdata($mo) [file root $resultfile].html
		flush $mo
		progresspage
	} else {
		set mo {}
	}
	return $mo
}

proc progresspage {{errormsg {}}} {
	global progresspage resultfile
	if {$progresspage} {
		file copy -force $resultfile.temp $resultfile
		set c [file_read [file root $resultfile].html.temp]
		if {$errormsg eq ""} {
			regsub {</head>} $c {<meta http-equiv="refresh" content="5"></head>} c
		}
		set tempo [open [file root $resultfile].html a]
		if {$errormsg ne ""} {
			append c "</table>\n"
			append c "<h3>Analysis stopped with an error</h3>\n$errormsg"
		} else {
			append c "<tr>\n"
			append c "<td>Running .... (refresh page for progress) </td>\n"
			append c "</tr>\n"
			append c "</table>\n"
		}
		append c "</body>\n</html>\n"
		file_write [file root $resultfile].html $c
	}
}

proc annotatemir_one_struct_fields {} {
	set result {
		Centroid_highest_impact MEA_highest_impact MFE_highest_impact 
		Centroid_impact MEA_impact MFE_impact
		ref_mfefreq var_mfefreq
		Cen_conservation MEA_conservation MFE_conservation
		ref_Cen_deltaG var_Cen_deltaG
		ref_MEA_deltaG var_MEA_deltaG
		ref_MFE_deltaG var_MFE_deltaG
	}
	lappend result 
	return $result
}

proc find_struct_changes {refstruct varstruct} {
	set changes {}
	set start -1
	set pos 0
	string_foreach r $refstruct v $varstruct {
		if {$r ne $v} {
			if {$start == -1} {
				set start $pos
			}
		} else {
			if {$start != -1} {
				lappend changes [list $start $pos]
				set start -1
			}
		}
		incr pos
	}
	if {$start != -1} {
		lappend changes [list $start $pos]
	}
	return $changes
}

proc VizAddCompar {img1 img2} {
	set html {}
	append html "<table class=\"mytable\">\n"
	append html "<tr><td><b>Reference</b></td><td><b>Variant</b></td></tr>\n"
	append html "<tr>\n"
	append html "		<td><a href=\"[file tail $img1]\">\n"
	append html "		<img class=structimg src=\"[file root [file tail $img1]].png\">\n"
	append html "		</a></td>\n"
	append html "		<td><a href=\"[file tail $img2]\">\n"
	append html "		<img class=structimg src=\"[file root [file tail $img2]].png\">\n"
	append html "		</a></td>\n"
	append html "</tr>\n"
	append html "</table>\n"
	return $html
}

proc annotatemir_one_struct_changes {geneobj changes} {
#puts ----------annotatemir_one_struct_changes----------
#putsvars geneobj changes
	set chromosome [chr_clip [dict get $geneobj chrom]]
	set complement [dict get $geneobj complement]
	set refseqstart [lindex [dict get $geneobj annotlist] 1 3]
	if {$complement} {
		set achanges [list_reverse $changes]
	} else {
		set achanges $changes
	}

	set prevannot {}
	set checkannot {}
	set annotloclist {}
	set structchanges {}
	set annotextra {}
	set genedel 0
	list_foreach {b e} $achanges {
		incr b $refseqstart
		incr e $refseqstart
		set oneloc [list $chromosome $b $e]
		foreach {impact mir} [annotatemir_one $oneloc $geneobj 0] break
# putsvars oneloc impact
		foreach impact [split $impact &] {
			if {$impact eq "GENEDEL"} {
				return deleted
			}
			if {![regexp {([^(]+\([a-z][+-])([0-9:e]+)\)(.*)} $impact temp annot annotloc extra]} {
				set annot $impact
				set annotloc {}
				set extra {}
			}
			if {$annot ne $checkannot} {
				if {$prevannot ne ""} {
					set annotloclist [list_remove $annotloclist {}]
					if {[llength $annotloclist]} {
						lappend structchanges ${prevannot}[join $annotloclist ,]\)$annotextra
					} else {
						lappend structchanges ${prevannot}$annotextra
					}
				}
				set prevannot $annot
				regsub : $prevannot {} checkannot
				set annotextra $extra
				set annotloclist [list $annotloc]
			} else {
				if {$extra ne ""} {set annotextra $extra}
				lappend annotloclist $annotloc
			}
		}
	}
	if {$prevannot ne ""} {
		lappend structchanges ${prevannot}[join $annotloclist ,]\)$annotextra
	}

}

proc annotatemir_one_struct_empty {loc mo {location {no miRNA within 2000 nt}}} {
	foreach {chromosome begin end type ref alt} $loc break
	if {$mo ne ""} {
		puts $mo "<tr>\n"
		puts $mo "<td>$type $chromosome:${begin}-${end} $ref>$alt</td>"
		puts $mo "<td>$location</td>\n"
		puts $mo "<td>variant not in analysed region</td>\n"
		puts $mo "<td></td>\n"
		puts $mo "</tr>\n"
	}
}

proc annotatemir_one_struct {loc geneobj resultfile mo mirvas locresult} {
#puts ----------annotatemir_one_struct----------
#putsvars loc geneobj resultfile mo mirvas locresult
	set result {}
	set location [lindex $locresult 0]
	if {$mo ne ""} {
		set outdir [file root $resultfile].structs
		file mkdir $outdir
	}
	set empty [list_fill [llength [annotatemir_one_struct_fields]] {}]
	array set a $geneobj
	set complement $a(complement)
	set mir $a(genename)
	foreach {chromosome begin end type ref alt} $loc break
	set varsize [expr {$end-$begin}]
	set altsize [string length $alt]
	set vardiff [expr {$altsize-$varsize}]
	# join $a(annotlist) \n
	set refseqstart [lindex $a(annotlist) 1 3]
	set refseqend [lindex $a(annotlist) end-1 4]
	set a(refseqstart) $refseqstart
	set a(refseqend) $refseqend
	set a(refseqsize) [expr {$refseqend - $refseqstart}]
	if {$end <= $refseqstart || $begin >= $refseqend} {
		if {$mo ne ""} {
			puts $mo "<tr>\n"
			puts $mo "<td>$mir $type $chromosome:${begin}-${end} $ref>$alt</td>"
			puts $mo "<td>$location</td>\n"
			puts $mo "<td>variant not in analysed region</td>\n"
			puts $mo "<td></td>\n"
			puts $mo "</tr>\n"
		}
		for {set i 0} {$i < 6} {incr i} {progress_next}
		return $empty
	}
	if {$begin > $refseqstart} {
		set varstart [expr {$begin - $refseqstart}]
	} else {
		set varstart 0
	}
	if {$end < $refseqend} {
		set varend [expr {$end - $refseqstart}]
	} else {
		set varend [expr {$refseqend - $refseqstart}]
	}
	# reference sequence
	set refseq [string toupper [genome_get $a(genomef) $chromosome $refseqstart $refseqend]]
	set refseqlen [string length $refseq]
	# reference sequence for hairpin only
	set hpseq [string toupper [genome_get $a(genomef) $chromosome $a(start) $a(end)]]
	if {![isint $ref]} {
		set checkref [string range $refseq $varstart [expr {$varend-1}]]
		if {$checkref ne $ref} {
			set msg "reference mismatch for $loc: found $checkref in stead of $ref"
			if {[catch {
				tk_messageBox -title "Input file error" -message $msg -icon error
			}]} {
				error $msg
			} 
		}
	}
	if {$mo ne ""} {
		if {[string length $alt] <= 10} {
			set ualt $alt
		} else {
			set ualt [string length $alt]
			if {[llength [glob -nocomplain ${outdir}/$mir\_$chromosome\_${begin}-${end}_$type\_$ref\_${ualt}\_*]]} {
				set num 2
				while {[llength [glob -nocomplain ${outdir}/$mir\_$chromosome\_${begin}-${end}_$type\_$ref\_${ualt}\_${num}\_*]]} {
					incr num
				}
				set ualt ${ualt}\_$num
			}
		}
		set outfilebase ${outdir}/$mir\_$chromosome\_${begin}-${end}_$type\_$ref\_${ualt}
	}
	# sequence with variant
	set varseq [string_replace $refseq $varstart [expr {$varend-1}] $alt]
	# the structures will be calculated on the sequence in the genes direction
	if {$complement} {
		set rnarefseq [seq_revcomplement $refseq]
		set rnahpseq [seq_revcomplement $hpseq]
		set rnavarseq [seq_revcomplement $varseq]
	} else {
		set rnarefseq $refseq
		set rnahpseq $hpseq
		set rnavarseq $varseq
	}
	regsub -all T $rnarefseq U rnarefseq
	regsub -all T $rnahpseq U rnahpseq
	regsub -all T $rnavarseq U rnavarseq
	if {$mo ne ""} {
		array set refa [getstructure $rnarefseq $outfilebase\_dot_Ref.ps]
		array set hpa [getstructure $rnahpseq $outfilebase\_dot_hpa.ps]
		array set vara [getstructure $rnavarseq $outfilebase\_dot_Var.ps]
	} else {
		array set refa [getstructure $rnarefseq]
		array set hpa [getstructure $rnahpseq]
		array set vara [getstructure $rnavarseq]
	}
	set tempresult {}
	set parthtml {}
	foreach structtype {Cen MEA MFE} {
		set refstruct $refa(struct,$structtype)
		set hpstruct $hpa(struct,$structtype)
		set varstruct $vara(struct,$structtype)
		# in order to get coordinates in (genome) forward direction allways, take reverse if gene is complenent
		if {$complement} {
			set refstruct [seq_revcomplement $refstruct]
			set hpstruct [seq_revcomplement $hpstruct]
			set varstruct [seq_revcomplement $varstruct]
		}
		# check if hairpin only structure is conserved
		set hpstart [expr {$a(start) - $refseqstart}]
		set hpend [expr {$a(end) - $refseqstart -1}]
		set hprefstruct [string range $refstruct $hpstart $hpend]
		if {$hprefstruct eq $hpstruct} {
			set resulta(cons,$structtype) conserved
		} else {
			set num 0
			string_foreach r $hprefstruct hp $hpstruct {
				if {$r ne $hp} {incr num}
			}
			set resulta(cons,$structtype) "changed $num"
		}
		# realign stuctures for indels
		if {$altsize < $varsize} {
			set varstruct [string_replace $varstruct $varstart -1 [string_fill X [expr {$varsize-$altsize}]]]
		} elseif {$altsize > $varsize} {
			set varstruct [string_replace $varstruct $varstart [expr {$varstart + $altsize - 1 - $varsize}] {}]
		}
		#
		# find changes (regions)
		set changes [find_struct_changes $refstruct $varstruct]
# putsvars loc geneobj structtype refstruct varstruct changes
		#
		# annotate structural changes
		set structchanges [annotatemir_one_struct_changes $geneobj $changes]
		if {[lindex $structchanges 0] in "genedel deleted"} {
			set test deleted
			set temp deleted
		} elseif {[llength $structchanges]} {
			set temp [join $structchanges &]
			foreach test {seed mature arm loop flank upstream} {if {[regexp $test $temp]} break}
		} else {
			set test nochange
			set temp nochange
		}
		set resulta(highestimpact,$structtype) $test
		set resulta(impact,$structtype) $temp
		lappend tempresult $test $temp
		#
		# draw structures
		if {$mo ne ""} {
			append parthtml "\n<h3>$structtype</h3>"
			if {$location eq "GENEDEL"} {
				append parthtml "<b>Gene $mir deleted ($type $chromosome:$begin-$end $ref > $alt)</b>\n"
			} elseif {![llength $structchanges]} {
				append parthtml "<b>No structural changes for $mir ($type $chromosome:$begin-$end $ref > $alt)</b>\n"
			} else {
				append parthtml "<b>Changes for $mir ($type $chromosome:$begin-$end $ref > $alt)</b>\n"
			}
			append parthtml [HTMLtable [list \
				[list {Hairpin conservation} $resulta(cons,$structtype)] \
				[list {Highest impact} $resulta(highestimpact,$structtype)] \
				[list {Impact} $resulta(impact,$structtype)] \
			]]
			# for drawing, we need the data in gene direction (not genomic forward)
			# coordinates of vars, changes are adapted to complement en var in the draw_structure proc
			log "drawing [file tail $outfilebase]\_${structtype}"
			set file1 "$outfilebase\_${structtype}_Ref"
			# foreach {out seq struct} [list $file1 $rnarefseq $refa(struct,$structtype)] break
			set changes [list_concat $changes]
			set img1 [draw_structure $file1 $rnarefseq $refa(struct,$structtype) a $type $varstart $varend 0 $changes]
			progress_next
			set file2 "$outfilebase\_${structtype}_Var"
			set img2 [draw_structure $file2 $rnavarseq $vara(struct,$structtype) a $type $varstart $varend $vardiff $changes]
			append parthtml [VizAddCompar $img1 $img2]
			progress_next
		} else {
			progress_next
		}
	}
	foreach structtype {Cen MEA MFE} {
		lappend result $resulta(highestimpact,$structtype)
	}
	foreach structtype {Cen MEA MFE} {
		lappend result $resulta(impact,$structtype)
	}
	lappend result $refa(mfefreq) $vara(mfefreq)
	foreach structtype {Cen MEA MFE} {
		lappend result $resulta(cons,$structtype)
	}
	foreach structtype {Cen MEA MFE} {
		lappend result $refa(DeltaG,$structtype) $vara(DeltaG,$structtype)
	}
	if {$mo ne ""} {
		# make html header
		set html {<body>}
		append html {
			<script type="text/javascript" src="setimgwidth.js"></script>
			<script>
			window.onload=setimgwidth
			</script>
			<link rel="stylesheet" type="text/css" href="grid.css" />
		}
		append html "<h1>microRNA: $mir</h1>\n"
		if {$complement} {set strand -} else {set strand +}
		append html [HTMLtable [list \
			[list microRNA $mir] \
			[list Strand $strand] \
			[list Variation	$type] \
			[list Chromosome $chromosome] \
			[list Position "${begin}-${end}"] \
			[list Change "$ref &gt; $alt"] \
			[list Location "$location"] \
		]]
		# add pregenerated html pointing to the structure drawings
		append html $parthtml
		# close and write
		append html "</body>"
		writehtml $outfilebase.html $html
		# add to list in mo
		set relativeoutfile [lindex [file split [file dir $outfilebase]] end]/[file tail $outfilebase].html
		puts $mo "<tr>\n"
		puts $mo "<td><a href=\"$relativeoutfile\">$mir $type $chromosome:${begin}-${end} $ref>$alt</a></td>"
		puts $mo "<td>$location</td>\n"
		puts $mo "<td>$resulta(highestimpact,Cen)<br>$resulta(highestimpact,MEA)<br>$resulta(highestimpact,MFE)</td>\n"
		puts $mo "<td>$resulta(cons,Cen)<br>$resulta(cons,MEA)<br>$resulta(cons,MFE)</td>\n"
		puts $mo "</tr>\n"
		flush $mo
		progresspage
	}
	return $result
}

proc mirvas_draw_close {mo} {
	if {$mo eq ""} return
	puts $mo "</table>\n</body>\n</html>"
	close $mo
	set moresultfile $::onestructdata($mo)
	file rename -force $moresultfile.temp $moresultfile
}

