proc java {} {
	global usejava java tcl_platform
	if {[info exists usejava]} {return $usejava}
	if {[info exists java]} {
		set usejava $java
	} elseif {$tcl_platform(platform) eq "unix"} {
		if {![catch {exec which java} usejava]} {
		} elseif {[info exists env(JAVA_HOME)] && ![catch {glob $env(JAVA_HOME)/bin/java} usejava]} {
		} elseif {[info exists env(JAVA_HOME)] && ![catch {glob $env(JAVA_HOME)/jre/bin/java} usejava]} {
		} elseif {![catch {glob /usr/java/bin/java} usejava]} {}
	} else {
		if {![catch {exec java.exe -version} version]} {
			# on the PATH
			set usejava java.exe
		} elseif {![catch {set usejava [findjava_win]}] && $usejava ne ""} {
			# gotten usejava through the registry
		} elseif {![catch {glob "C:/Program Files/Java/*/bin/java.exe"} list]} {
			set usejava [lindex [lsort -dict $list] end]
		} elseif {![catch {glob "D:/Program Files/Java/*/bin/java.exe"} list]} {
			set usejava [lindex [lsort -dict $list] end]
		} else {
			error "Could not find java. You can set the location of the java executable in the file settings.txt"
		}
	}
	set version {}
	catch {exec $usejava -version} version
	if {![regexp ^java $version]}	{
		error "Could not find java, \"$usejava\" does not return a proper version string on -version option"
	}
	return $usejava
}

# code from http://wiki.tcl.tk/11294 by Emmanuel Frecon
proc findjava_win {} {
     package require registry

     proc __jre_get { jre_root { vnumber_p "" } { vnum_match "*" } } {
 	set java_home ""

 	if { $vnumber_p != "" } {
 	    upvar $vnumber_p vnumber
 	}
 	set res [catch [list registry get "$jre_root" CurrentVersion] vnumber]
 	if { $res == 0 && [string match $vnum_match $vnumber] } {
 	    catch [list registry get "${jre_root}\\${vnumber}" JavaHome] java_home
 	} else {
 	    foreach vnumber [registry keys $jre_root] {
 		if { [string match $vnum_match $vnumber] } {
 		    set res [catch [list registry get "${jre_root}\\${vnumber}" \
 					JavaHome] java_home]
 		    if { $res == 0 } {
 			break
 		    }
 		}
 	    }
 	}
 	return $java_home
     }

     proc __jre_exec { jre_root { vnumber_p "" } { vnum_match "*" } } {
 	if { $vnumber_p != "" } {
 	    upvar $vnumber_p vnumber
 	}
 	set j_home [__jre_get $jre_root vnumber $vnum_match]
 	if { $j_home != "" } {
 	    # Make a Tcl-friendly path of the horrible windows backslashes
 	    set j_home [regsub -all "\\\\" $j_home "/"]
 	    set j_exe [file join $j_home "bin" "java.exe"]
 	    if { [file executable $j_exe] } {
 		return $j_exe
 	    }
 	}
 	return ""
     }

     proc java_find { { vnumber_p "" } { vnum_match "*" } } {
 	if { $vnumber_p != "" } {
 	    upvar $vnumber_p vnumber
 	}
 	set vnumber ""
 	set root "HKEY_LOCAL_MACHINE\\Software\\JavaSoft"
 	set j_exe [__jre_exec "$root\\Java Runtime Environment" vnumber $vnum_match]
 	if { $j_exe == "" } {
 	    set j_exe [__jre_exec "$root\\Java Development Kit" vnumber $vnum_match]
 	    if { $j_exe == "" } {
 		set root "HKEY_LOCAL_MACHINE\\Software\\IBM"
 		set j_exe [__jre_exec "$root\\Java Runtime Environment" vnumber $vnum_match]
 		if { $j_exe == "" } {
 		    set j_exe [__jre_exec "$root\\Java Development Kit" vnumber $vnum_match]
 		}
 	    }
 	}
 	return $j_exe
     }

     java_find
}
