
catch {rename bgerror bgerror.old}
proc bgerror {message} {
	::tk_messageBox -icon error -message $message -title Error
	puts stderr $::errorInfo
	if {[info exists ::tcl_dirtcl]} {set dir $::tcl_dirtcl} else {set dir $::appdir}
	file_write [file join $dir errorlog.txt] $::errorInfo
}

proc lpop {stackVar} {
	upvar $stackVar stack
	set result [lindex $stack end]
	set stack [lrange $stack 0 end-1]
	return $result
}

proc seq_revcomplement {seq} {
	array set compla {A T T A G C C G ( ) ) ( . .}
	set cseq {}
	for {set i [expr {[string length $seq]-1}]} {$i >= 0} {incr i -1} {
		append cseq $compla([string index $seq $i])
	}
	return $cseq
}

proc string_replace {string s e replacestring} {
	if {$e < $s} {
		set e $s
	} else {
		set e [expr {$e+1}]
	}
	return [string range $string 0 [expr {$s-1}]]$replacestring[string range $string $e end]
}

proc findloop {struct} {
	unset -nocomplain structa
	set sstruct [split $struct ""]
	set stack {}
	set pos 0
	set helix 0
	set climbing 1
	foreach ch $sstruct {
		if {$ch eq "\("} {
			if {!$climbing} {
				incr helix
				lappend stack switch
			}
			lappend stack $pos
		} elseif {$ch eq "\)"} {
			set s [lpop stack]
			if {$climbing || $s eq "switch"} {
				incr helix
				set climbing 0
				while {$s eq "switch"} {
					set s [lpop stack]
				}
			}
			lappend structa($helix) [list $s $pos]
		}
		incr pos
	}
	foreach {s e} [lindex $structa($helix) 0] break
	incr s ; incr e -1
	return [list $s $e]
}

proc reload {} {
	set files [lsort -dict [glob $::appdir/lib/*.tcl]]
	set errors {}
	set result {}
	foreach file $files {
		append result "Loading $file\n"
		if [catch {source $file} error] {
			append errors "$file: $error\n"
		}
	}
	if {$errors ne ""} {
		append result "\n*****Errors*****\n$errors"
	}
	return $result
}

proc log message {
	puts stderr $message
	catch {
		.top.log configure -text $message
		update
	}
}

proc progress_step {args} {
	if {[llength $args]} {
		set ::progress_step [lindex $args 0]
	} else {
		return $::progress_step
	}
}

proc progress_next {} {
	catch {
		.top.progress configure -value [expr {[.top.progress cget -value] + $::progress_step}]
		update
	} e
}

proc progress_set {pos} {
	catch {
		.top.progress configure -value $pos
		update
	} e
}

proc progress_max {{max -1} {pos 0}} {
	if {$max == -1} {
		catch {
			set max [.top.progress cget -max]
		} e
		return $max
	} else {
		catch {
			.top.progress configure -max $max
			.top.progress configure -value $pos
			update
		} e
	}
}

proc progress_cur {} {
	set pos 0
	catch {
		set pos [.top.progress cget -value]
	} e
	return $pos
}

proc launchBrowser url {
    global tcl_platform
    if {$tcl_platform(platform) eq "windows"} {
        set command [list {*}[auto_execok start] {}]
        # Windows shell would start a new command after &, so shell escape it with ^
        set url [string map {& ^&} $url]
    } elseif {$tcl_platform(os) eq "Darwin"} {
        # It *is* generally a mistake to use $tcl_platform(os) to select functionality,
        # particularly in comparison to $tcl_platform(platform).  For now, let's just
        # regard it as a stylistic variation subject to debate.
        set command [list open]
    } else {
        set command [list xdg-open]
    }
    exec {*}$command $url &
}


if 0 {
set struct ((((((....))))))
findloop $struct
set struct ((((((....))((....))))))
findloop $struct
set struct (((..)(..))(..))
findloop $struct
}
