#!/bin/sh
# the next line restarts using wish \
exec tclsh "$0" ${1+"$@"}

# see where we are, add the lib dir in the original location to auto_path, load extension
set script [file join [pwd] [info script]]
set scriptname [file tail $script]
while 1 {
	if {[catch {set script [file join [pwd] [file readlink $script]]}]} break
}

package require Extral
set appdir [file dir $script]
lappend auto_path $appdir/lib

set dir $appdir

if {[info exists tcl_dirtcl] && [file exists $tcl_dirtcl/settings.txt]} {
	source $tcl_dirtcl/settings.txt
} elseif {[file exists $appdir/settings.txt]} {
	source $appdir/settings.txt
}

# gui on no args
if {[llength $argv] < 1} {
	package require Tk
	GUI $appdir
} elseif {[lindex $argv 0] eq "convert"} {
	set structfile {}
	foreach {gff3file resultfile genomefile structfile} [lrange $argv 1 end] break
	convertmirbase $gff3file $resultfile $genomefile $structfile
} else {
	set genecol name
	set transcriptcol transcript
	set progresspage 0
	set flank5p {}
	set flank3p {}
	set drawings 1
	set isomirname 0
	set pos 0
	foreach {key value} $argv {
		switch -- $key {
			-genecol {
				set genecol $value
			}
			-transcriptcol {
				set transcriptcol $value
			}
			-flank5p {
				set flank5p $value
			}
			-flank3p {
				set flank3p $value
			}
			-drawings {
				set drawings $value
			}
			-isomirname {
				set isomirname $value
			}
			-progresspage {
				set progresspage $value
			}
			default {
				break
			}
		}
		incr pos 2
	}
	if {$flank5p eq ""} {
		if {$flank3p eq ""} {
			set flanksizes {100 100}
		} else {
			set flanksizes [list $flank3p $flank3p]
		}
	} elseif {$flank3p eq ""} {
		set flanksizes [list $flank5p $flank5p]
	} else {
		set flanksizes [list $flank5p $flank3p]
	}
	set argv [lrange $argv $pos end]
	if {[llength $argv] < 4} {
		puts stderr {error in format: format is:
mirvas ?options? varfile resultfile genomefile dbfile ...
Possible options are: -flank5p, -flank3p, -drawings
or
mirvas convertmirbase gff3file resultfile genomefile ?structfile?
to create a miRNA database from mirbase data
}
		exit 1
	}
	if {$drawings} {set mirvas 2} else {set mirvas 1}
	foreach {file resultfile genomefile dbfile} $argv break
	set dbfiles [lrange $argv 4 end]
	set file [file normalize $file]
	set genomefile [file normalize $genomefile]
	set dbfile [file normalize $dbfile]
	set resultfile [file normalize $resultfile]
	puts stderr "file=$file"
	puts stderr "genomefile=$genomefile"
	puts stderr "dbfile=$dbfile"
	puts stderr "resultfile=$resultfile"
	puts stderr "flank5p=[lindex $flanksizes 0]"
	puts stderr "flank3p=[lindex $flanksizes 1]"
	puts stderr "isomirname=$isomirname"
	set file [convert_if_vcf $file]
	if {!$progresspage} {
		annotatemir $file $genomefile $dbfile mir $resultfile $genecol $transcriptcol $flanksizes $isomirname $mirvas
	} elseif {[catch {
		annotatemir $file $genomefile $dbfile mir $resultfile $genecol $transcriptcol $flanksizes $isomirname $mirvas
	} errormsg]} {
		progresspage $errormsg
	}
}
